@@ -37,10 +37,7 @@
 #include "util/u_cpu_detect.h"
 
 /*
- * TGSI translation limits.
- *
- * Some are slightly above SM 3.0 requirements to give some wiggle room to
- * the gallium frontends.
+ * llvmpipe shader limits
  */
 
 #define LP_MAX_TGSI_TEMPS 4096
@@ -55,11 +52,11 @@
 
 #define LP_MAX_TGSI_CONST_BUFFER_SIZE (LP_MAX_TGSI_CONSTS * sizeof(float[4]))
 
-#define LP_MAX_TGSI_SHADER_BUFFERS 16
+#define LP_MAX_TGSI_SHADER_BUFFERS 32
 
 #define LP_MAX_TGSI_SHADER_BUFFER_SIZE (1 << 27)
 
-#define LP_MAX_TGSI_SHADER_IMAGES 32
+#define LP_MAX_TGSI_SHADER_IMAGES 64
 
 /*
  * For quick access we cache registers in statically
@@ -89,7 +86,7 @@
 static inline bool
 lp_has_fp16(void)
 {
-   return util_get_cpu_caps()->has_f16c;
+   return util_get_cpu_caps()->has_f16c || DETECT_ARCH_AARCH64;
 }
 
 /**
@@ -145,16 +142,11 @@ gallivm_get_shader_param(enum pipe_shader_cap param)
       return PIPE_MAX_SAMPLERS;
    case PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS:
       return PIPE_MAX_SHADER_SAMPLER_VIEWS;
-   case PIPE_SHADER_CAP_PREFERRED_IR:
-      return PIPE_SHADER_IR_TGSI;
    case PIPE_SHADER_CAP_SUPPORTED_IRS:
       return (1 << PIPE_SHADER_IR_TGSI) | (1 << PIPE_SHADER_IR_NIR);
    case PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED:
    case PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE:
       return 1;
-   case PIPE_SHADER_CAP_DROUND_SUPPORTED:
-   case PIPE_SHADER_CAP_DFRACEXP_DLDEXP_SUPPORTED:
-   case PIPE_SHADER_CAP_LDEXP_SUPPORTED:
    case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS:
    case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS:
       return 0;
