@@ -66,9 +66,13 @@ EXTENSIONS = [
     Extension("VK_KHR_maintenance4",
               alias="maint4",
               features=True),
+    Extension("VK_KHR_maintenance5",
+              alias="maint5",
+              features=True, properties=True),
     Extension("VK_KHR_external_memory"),
     Extension("VK_KHR_external_memory_fd"),
     Extension("VK_KHR_vulkan_memory_model"),
+    Extension("VK_KHR_workgroup_memory_explicit_layout", alias="explicit_layout", features=True),
     Extension("VK_KHR_pipeline_executable_properties",
               alias="pipestats",
               features=True),
@@ -84,6 +88,7 @@ EXTENSIONS = [
     Extension("VK_KHR_buffer_device_address",
               alias="bda",
               features=True),
+    Extension("VK_EXT_external_memory_host", alias="ext_host_mem", properties=True),
     Extension("VK_EXT_queue_family_foreign"),
     Extension("VK_KHR_swapchain_mutable_format"),
     Extension("VK_EXT_provoking_vertex",
@@ -123,10 +128,16 @@ EXTENSIONS = [
               alias="driver",
               properties=True),
     Extension("VK_EXT_memory_budget"),
+    Extension("VK_EXT_memory_priority", alias="memprio", features=True),
+    Extension("VK_EXT_pageable_device_local_memory", alias="mempage", features=True),
     Extension("VK_KHR_draw_indirect_count"),
+    Extension("VK_EXT_dynamic_rendering_unused_attachments", alias="unused", features=True),
+    Extension("VK_EXT_shader_object", alias="shobj", features=True, properties=True),
     Extension("VK_EXT_attachment_feedback_loop_layout",
               alias="feedback_loop",
               features=True),
+    Extension("VK_EXT_attachment_feedback_loop_dynamic_state", alias="feedback_dyn", features=True),
+    Extension("VK_NV_device_generated_commands", alias="nv_dgc", features=True, properties=True),
     Extension("VK_EXT_fragment_shader_interlock",
               alias="interlock",
               features=True,
@@ -134,10 +145,6 @@ EXTENSIONS = [
     Extension("VK_EXT_sample_locations",
               alias="sample_locations",
               properties=True),
-    Extension("VK_EXT_conservative_rasterization",
-              alias="cons_raster",
-              properties=True,
-              conditions=["$props.fullyCoveredFragmentShaderInputVariable"]),
     Extension("VK_KHR_shader_draw_parameters"),
     Extension("VK_KHR_sampler_mirror_clamp_to_edge"),
     Extension("VK_EXT_descriptor_buffer", alias="db", features=True, properties=True),
@@ -188,6 +195,10 @@ EXTENSIONS = [
               alias="shader_clock",
               features=True,
               conditions=["$feats.shaderSubgroupClock"]),
+    Extension("VK_INTEL_shader_integer_functions2",
+              alias="shader_int_fns2",
+              features=True,
+              conditions=["$feats.shaderIntegerFunctions2"]),
     Extension("VK_EXT_sampler_filter_minmax",
               alias="reduction",
               properties=True,
@@ -243,6 +254,10 @@ EXTENSIONS = [
     Extension("VK_EXT_4444_formats",
               alias="format_4444",
               features=True),
+    Extension("VK_EXT_host_image_copy",
+              alias="hic",
+              features=True,
+              properties=True),
     Extension("VK_EXT_scalar_block_layout",
               alias="scalar_block_layout",
               features=True,
@@ -299,6 +314,8 @@ EXTENSIONS = [
               alias="demote",
               features=True,
               conditions=["$feats.shaderDemoteToHelperInvocation"]),
+    Extension("VK_KHR_shader_float_controls",
+              alias="float_controls")
 ]
 
 # constructor: Versions(device_version(major, minor, patch), struct_version(major, minor))
@@ -773,12 +790,12 @@ if __name__ == "__main__":
     lookup = TemplateLookup()
     lookup.put_string("helpers", include_template)
 
-    with open(header_path, "w") as header_file:
+    with open(header_path, "w", encoding='utf-8') as header_file:
         header = Template(header_code, lookup=lookup).render(extensions=extensions, versions=versions, registry=registry).strip()
         header = replace_code(header, replacement)
         print(header, file=header_file)
 
-    with open(impl_path, "w") as impl_file:
+    with open(impl_path, "w", encoding='utf-8') as impl_file:
         impl = Template(impl_code, lookup=lookup).render(extensions=extensions, versions=versions, registry=registry).strip()
         impl = replace_code(impl, replacement)
         print(impl, file=impl_file)
