@@ -523,6 +523,7 @@ static const char *const sync_function[16] = {
    [TGL_SYNC_NOP] = "nop",
    [TGL_SYNC_ALLRD] = "allrd",
    [TGL_SYNC_ALLWR] = "allwr",
+   [TGL_SYNC_FENCE] = "fence",
    [TGL_SYNC_BAR] = "bar",
    [TGL_SYNC_HOST] = "host",
 };
@@ -754,6 +755,37 @@ static const char* const lsc_cache_store[] = {
    [LSC_CACHE_STORE_L1WB_L3WB]       = "L1WB_L3WB",
 };
 
+static const char* const xe2_lsc_cache_load[] = {
+   [XE2_LSC_CACHE_LOAD_L1STATE_L3MOCS]   = "L1STATE_L3MOCS",
+   [XE2_LSC_CACHE_LOAD_L1UC_L3UC]        = "L1UC_L3UC",
+   [XE2_LSC_CACHE_LOAD_L1UC_L3C]         = "L1UC_L3C",
+   [XE2_LSC_CACHE_LOAD_L1UC_L3CC]        = "L1UC_L3CC",
+   [XE2_LSC_CACHE_LOAD_L1C_L3UC]         = "L1C_L3UC",
+   [XE2_LSC_CACHE_LOAD_L1C_L3C]          = "L1C_L3C",
+   [XE2_LSC_CACHE_LOAD_L1C_L3CC]         = "L1C_L3CC",
+   [XE2_LSC_CACHE_LOAD_L1S_L3UC]         = "L1S_L3UC",
+   [XE2_LSC_CACHE_LOAD_L1S_L3C]          = "L1S_L3C",
+   [XE2_LSC_CACHE_LOAD_L1IAR_L3IAR]      = "L1IAR_L3IAR",
+};
+
+static const char* const xe2_lsc_cache_store[] = {
+   [XE2_LSC_CACHE_STORE_L1STATE_L3MOCS]  = "L1STATE_L3MOCS",
+   [XE2_LSC_CACHE_STORE_L1UC_L3UC]       = "L1UC_L3UC",
+   [XE2_LSC_CACHE_STORE_L1UC_L3WB]       = "L1UC_L3WB",
+   [XE2_LSC_CACHE_STORE_L1WT_L3UC]       = "L1WT_L3UC",
+   [XE2_LSC_CACHE_STORE_L1WT_L3WB]       = "L1WT_L3WB",
+   [XE2_LSC_CACHE_STORE_L1S_L3UC]        = "L1S_L3UC",
+   [XE2_LSC_CACHE_STORE_L1S_L3WB]        = "L1S_L3WB",
+   [XE2_LSC_CACHE_STORE_L1WB_L3WB]       = "L1WB_L3WB",
+};
+
+static const char* const dpas_systolic_depth[4] = {
+   [0] = "16",
+   [1] = "2",
+   [2] = "4",
+   [3] = "8"
+};
+
 static int column;
 
 static int
@@ -816,10 +848,10 @@ control(FILE *file, const char *name, const char *const ctrl[],
 }
 
 static int
-print_opcode(FILE *file, const struct intel_device_info *devinfo,
+print_opcode(FILE *file, const struct brw_isa_info *isa,
              enum opcode id)
 {
-   const struct opcode_desc *desc = brw_opcode_desc(devinfo, id);
+   const struct opcode_desc *desc = brw_opcode_desc(isa, id);
    if (!desc) {
       format(file, "*** invalid opcode value %d ", id);
       return 1;
@@ -891,13 +923,14 @@ reg(FILE *file, unsigned _reg_file, unsigned _reg_nr)
 }
 
 static int
-dest(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
+dest(FILE *file, const struct brw_isa_info *isa, const brw_inst *inst)
 {
+   const struct intel_device_info *devinfo = isa->devinfo;
    enum brw_reg_type type = brw_inst_dst_type(devinfo, inst);
    unsigned elem_size = brw_reg_type_to_size(type);
    int err = 0;
 
-   if (is_split_send(devinfo, brw_inst_opcode(devinfo, inst))) {
+   if (is_split_send(devinfo, brw_inst_opcode(isa, inst))) {
       /* These are fixed for split sends */
       type = BRW_REGISTER_TYPE_UD;
       elem_size = 4;
@@ -1018,6 +1051,27 @@ dest_3src(FILE *file, const struct intel_device_info *devinfo,
    return 0;
 }
 
+static int
+dest_dpas_3src(FILE *file, const struct intel_device_info *devinfo,
+               const brw_inst *inst)
+{
+   uint32_t reg_file = brw_inst_dpas_3src_dst_reg_file(devinfo, inst);
+
+   if (reg(file, reg_file, brw_inst_dpas_3src_dst_reg_nr(devinfo, inst)) == -1)
+      return 0;
+
+   enum brw_reg_type type = brw_inst_dpas_3src_dst_type(devinfo, inst);
+   unsigned subreg_nr = brw_inst_dpas_3src_dst_subreg_nr(devinfo, inst);
+
+   if (subreg_nr)
+      format(file, ".%u", subreg_nr);
+   string(file, "<1>");
+
+   string(file, brw_reg_type_to_letters(type));
+
+   return 0;
+}
+
 static int
 src_align1_region(FILE *file,
                   unsigned _vert_stride, unsigned _width,
@@ -1216,6 +1270,7 @@ implied_width(enum brw_vertical_stride _vert_stride,
    /* "2. Width is equal to vertical stride when Horizontal Stride is zero." */
    } else if (_horiz_stride == BRW_HORIZONTAL_STRIDE_0) {
       switch (_vert_stride) {
+      case BRW_VERTICAL_STRIDE_1: return BRW_WIDTH_1;
       case BRW_VERTICAL_STRIDE_2: return BRW_WIDTH_2;
       case BRW_VERTICAL_STRIDE_4: return BRW_WIDTH_4;
       case BRW_VERTICAL_STRIDE_8: return BRW_WIDTH_8;
@@ -1492,9 +1547,83 @@ src2_3src(FILE *file, const struct intel_device_info *devinfo,
 }
 
 static int
-imm(FILE *file, const struct intel_device_info *devinfo, enum brw_reg_type type,
+src0_dpas_3src(FILE *file, const struct intel_device_info *devinfo,
+               const brw_inst *inst)
+{
+   uint32_t reg_file = brw_inst_dpas_3src_src0_reg_file(devinfo, inst);
+
+   if (reg(file, reg_file, brw_inst_dpas_3src_src0_reg_nr(devinfo, inst)) == -1)
+      return 0;
+
+   unsigned subreg_nr = brw_inst_dpas_3src_src0_subreg_nr(devinfo, inst);
+   enum brw_reg_type type = brw_inst_dpas_3src_src0_type(devinfo, inst);
+
+   if (subreg_nr)
+      format(file, ".%d", subreg_nr);
+   src_align1_region(file,
+                     BRW_VERTICAL_STRIDE_1,
+                     BRW_WIDTH_1,
+                     BRW_ALIGN1_3SRC_SRC_HORIZONTAL_STRIDE_0);
+
+   string(file, brw_reg_type_to_letters(type));
+
+   return 0;
+}
+
+static int
+src1_dpas_3src(FILE *file, const struct intel_device_info *devinfo,
+               const brw_inst *inst)
+{
+   uint32_t reg_file = brw_inst_dpas_3src_src1_reg_file(devinfo, inst);
+
+   if (reg(file, reg_file, brw_inst_dpas_3src_src1_reg_nr(devinfo, inst)) == -1)
+      return 0;
+
+   unsigned subreg_nr = brw_inst_dpas_3src_src1_subreg_nr(devinfo, inst);
+   enum brw_reg_type type = brw_inst_dpas_3src_src1_type(devinfo, inst);
+
+   if (subreg_nr)
+      format(file, ".%d", subreg_nr);
+   src_align1_region(file,
+                     BRW_VERTICAL_STRIDE_1,
+                     BRW_WIDTH_1,
+                     BRW_ALIGN1_3SRC_SRC_HORIZONTAL_STRIDE_0);
+
+   string(file, brw_reg_type_to_letters(type));
+
+   return 0;
+}
+
+static int
+src2_dpas_3src(FILE *file, const struct intel_device_info *devinfo,
+               const brw_inst *inst)
+{
+   uint32_t reg_file = brw_inst_dpas_3src_src2_reg_file(devinfo, inst);
+
+   if (reg(file, reg_file, brw_inst_dpas_3src_src2_reg_nr(devinfo, inst)) == -1)
+      return 0;
+
+   unsigned subreg_nr = brw_inst_dpas_3src_src2_subreg_nr(devinfo, inst);
+   enum brw_reg_type type = brw_inst_dpas_3src_src2_type(devinfo, inst);
+
+   if (subreg_nr)
+      format(file, ".%d", subreg_nr);
+   src_align1_region(file,
+                     BRW_VERTICAL_STRIDE_1,
+                     BRW_WIDTH_1,
+                     BRW_ALIGN1_3SRC_SRC_HORIZONTAL_STRIDE_0);
+
+   string(file, brw_reg_type_to_letters(type));
+
+   return 0;
+}
+
+static int
+imm(FILE *file, const struct brw_isa_info *isa, enum brw_reg_type type,
     const brw_inst *inst)
 {
+   const struct intel_device_info *devinfo = isa->devinfo;
+
    switch (type) {
    case BRW_REGISTER_TYPE_UQ:
       format(file, "0x%016"PRIx64"UQ", brw_inst_imm_uq(devinfo, inst));
@@ -1533,7 +1662,7 @@ imm(FILE *file, const struct intel_device_info *devinfo, enum brw_reg_type type,
       /* The DIM instruction's src0 uses an F type but contains a
        * 64-bit immediate
        */
-      if (brw_inst_opcode(devinfo, inst) == BRW_OPCODE_DIM) {
+      if (brw_inst_opcode(isa, inst) == BRW_OPCODE_DIM) {
          format(file, "0x%"PRIx64"F", brw_inst_bits(inst, 127, 64));
          pad(file, 48);
          format(file, "/* %-gF */", brw_inst_imm_df(devinfo, inst));
@@ -1544,12 +1673,16 @@ imm(FILE *file, const struct intel_device_info *devinfo, enum brw_reg_type type,
       }
       break;
    case BRW_REGISTER_TYPE_DF:
-      format(file, "0x%016"PRIx64"DF", brw_inst_bits(inst, 127, 64));
+      format(file, "0x%016"PRIx64"DF", brw_inst_imm_uq(devinfo, inst));
       pad(file, 48);
       format(file, "/* %-gDF */", brw_inst_imm_df(devinfo, inst));
       break;
    case BRW_REGISTER_TYPE_HF:
-      string(file, "Half Float IMM");
+      format(file, "0x%04xHF",
+             (uint16_t) brw_inst_imm_ud(devinfo, inst));
+      pad(file, 48);
+      format(file, "/* %-gHF */",
+             _mesa_half_to_float((uint16_t) brw_inst_imm_ud(devinfo, inst)));
       break;
    case BRW_REGISTER_TYPE_NF:
    case BRW_REGISTER_TYPE_UB:
@@ -1611,9 +1744,11 @@ src_send_desc_ia(FILE *file,
 }
 
 static int
-src0(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
+src0(FILE *file, const struct brw_isa_info *isa, const brw_inst *inst)
 {
-   if (is_split_send(devinfo, brw_inst_opcode(devinfo, inst))) {
+   const struct intel_device_info *devinfo = isa->devinfo;
+
+   if (is_split_send(devinfo, brw_inst_opcode(isa, inst))) {
       if (devinfo->ver >= 12) {
          return src_sends_da(file,
                              devinfo,
@@ -1636,12 +1771,12 @@ src0(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
                              brw_inst_src0_ia_subreg_nr(devinfo, inst));
       }
    } else if (brw_inst_src0_reg_file(devinfo, inst) == BRW_IMMEDIATE_VALUE) {
-      return imm(file, devinfo, brw_inst_src0_type(devinfo, inst), inst);
+      return imm(file, isa, brw_inst_src0_type(devinfo, inst), inst);
    } else if (brw_inst_access_mode(devinfo, inst) == BRW_ALIGN_1) {
       if (brw_inst_src0_address_mode(devinfo, inst) == BRW_ADDRESS_DIRECT) {
          return src_da1(file,
                         devinfo,
-                        brw_inst_opcode(devinfo, inst),
+                        brw_inst_opcode(isa, inst),
                         brw_inst_src0_type(devinfo, inst),
                         brw_inst_src0_reg_file(devinfo, inst),
                         brw_inst_src0_vstride(devinfo, inst),
@@ -1654,7 +1789,7 @@ src0(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
       } else {
          return src_ia1(file,
                         devinfo,
-                        brw_inst_opcode(devinfo, inst),
+                        brw_inst_opcode(isa, inst),
                         brw_inst_src0_type(devinfo, inst),
                         brw_inst_src0_ia1_addr_imm(devinfo, inst),
                         brw_inst_src0_ia_subreg_nr(devinfo, inst),
@@ -1668,7 +1803,7 @@ src0(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
       if (brw_inst_src0_address_mode(devinfo, inst) == BRW_ADDRESS_DIRECT) {
          return src_da16(file,
                          devinfo,
-                         brw_inst_opcode(devinfo, inst),
+                         brw_inst_opcode(isa, inst),
                          brw_inst_src0_type(devinfo, inst),
                          brw_inst_src0_reg_file(devinfo, inst),
                          brw_inst_src0_vstride(devinfo, inst),
@@ -1688,9 +1823,11 @@ src0(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
 }
 
 static int
-src1(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
+src1(FILE *file, const struct brw_isa_info *isa, const brw_inst *inst)
 {
-   if (is_split_send(devinfo, brw_inst_opcode(devinfo, inst))) {
+   const struct intel_device_info *devinfo = isa->devinfo;
+
+   if (is_split_send(devinfo, brw_inst_opcode(isa, inst))) {
       return src_sends_da(file,
                           devinfo,
                           BRW_REGISTER_TYPE_UD,
@@ -1698,12 +1835,12 @@ src1(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
                           brw_inst_send_src1_reg_nr(devinfo, inst),
                           0 /* subreg_nr */);
    } else if (brw_inst_src1_reg_file(devinfo, inst) == BRW_IMMEDIATE_VALUE) {
-      return imm(file, devinfo, brw_inst_src1_type(devinfo, inst), inst);
+      return imm(file, isa, brw_inst_src1_type(devinfo, inst), inst);
    } else if (brw_inst_access_mode(devinfo, inst) == BRW_ALIGN_1) {
       if (brw_inst_src1_address_mode(devinfo, inst) == BRW_ADDRESS_DIRECT) {
          return src_da1(file,
                         devinfo,
-                        brw_inst_opcode(devinfo, inst),
+                        brw_inst_opcode(isa, inst),
                         brw_inst_src1_type(devinfo, inst),
                         brw_inst_src1_reg_file(devinfo, inst),
                         brw_inst_src1_vstride(devinfo, inst),
@@ -1716,7 +1853,7 @@ src1(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
       } else {
          return src_ia1(file,
                         devinfo,
-                        brw_inst_opcode(devinfo, inst),
+                        brw_inst_opcode(isa, inst),
                         brw_inst_src1_type(devinfo, inst),
                         brw_inst_src1_ia1_addr_imm(devinfo, inst),
                         brw_inst_src1_ia_subreg_nr(devinfo, inst),
@@ -1730,7 +1867,7 @@ src1(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
       if (brw_inst_src1_address_mode(devinfo, inst) == BRW_ADDRESS_DIRECT) {
          return src_da16(file,
                          devinfo,
-                         brw_inst_opcode(devinfo, inst),
+                         brw_inst_opcode(isa, inst),
                          brw_inst_src1_type(devinfo, inst),
                          brw_inst_src1_reg_file(devinfo, inst),
                          brw_inst_src1_vstride(devinfo, inst),
@@ -1755,7 +1892,7 @@ qtr_ctrl(FILE *file, const struct intel_device_info *devinfo,
 {
    int qtr_ctl = brw_inst_qtr_control(devinfo, inst);
    int exec_size = 1 << brw_inst_exec_size(devinfo, inst);
-   const unsigned nib_ctl = devinfo->ver < 7 ? 0 :
+   const unsigned nib_ctl = devinfo->ver < 7 || devinfo->ver >= 20 ? 0 :
                             brw_inst_nib_control(devinfo, inst);
 
    if (exec_size < 8 || nib_ctl) {
@@ -1784,12 +1921,44 @@ qtr_ctrl(FILE *file, const struct intel_device_info *devinfo,
    return 0;
 }
 
-static int
-swsb(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
+static bool
+inst_has_type(const struct brw_isa_info *isa,
+              const brw_inst *inst,
+              enum brw_reg_type type)
 {
-   const enum opcode opcode = brw_inst_opcode(devinfo, inst);
-   const uint8_t x = brw_inst_swsb(devinfo, inst);
-   const struct tgl_swsb swsb = tgl_swsb_decode(devinfo, opcode, x);
+   const struct intel_device_info *devinfo = isa->devinfo;
+   const unsigned num_sources = brw_num_sources_from_inst(isa, inst);
+
+   if (brw_inst_dst_type(devinfo, inst) == type)
+      return true;
+
+   if (num_sources >= 3) {
+      if (brw_inst_3src_access_mode(devinfo, inst) == BRW_ALIGN_1)
+         return brw_inst_3src_a1_src0_type(devinfo, inst) == type ||
+                brw_inst_3src_a1_src1_type(devinfo, inst) == type ||
+                brw_inst_3src_a1_src2_type(devinfo, inst) == type;
+      else
+         return brw_inst_3src_a16_src_type(devinfo, inst) == type;
+   } else if (num_sources == 2) {
+      return brw_inst_src0_type(devinfo, inst) == type ||
+             brw_inst_src1_type(devinfo, inst) == type;
+   } else {
+      return brw_inst_src0_type(devinfo, inst) == type;
+   }
+}
+
+static int
+swsb(FILE *file, const struct brw_isa_info *isa, const brw_inst *inst)
+{
+   const struct intel_device_info *devinfo = isa->devinfo;
+   const enum opcode opcode = brw_inst_opcode(isa, inst);
+   const uint32_t x = brw_inst_swsb(devinfo, inst);
+   const bool is_unordered =
+      opcode == BRW_OPCODE_SEND || opcode == BRW_OPCODE_SENDC ||
+      opcode == BRW_OPCODE_MATH || opcode == BRW_OPCODE_DPAS ||
+      (devinfo->has_64bit_float_via_math_pipe &&
+       inst_has_type(isa, inst, BRW_REGISTER_TYPE_DF));
+   const struct tgl_swsb swsb = tgl_swsb_decode(devinfo, is_unordered, x);
    if (swsb.regdist)
       format(file, " %s@%d",
              (swsb.pipe == TGL_PIPE_FLOAT ? "F" :
@@ -1806,13 +1975,13 @@ swsb(FILE *file, const struct intel_device_info *devinfo, const brw_inst *inst)
 
 #ifdef DEBUG
 static __attribute__((__unused__)) int
-brw_disassemble_imm(const struct intel_device_info *devinfo,
+brw_disassemble_imm(const struct brw_isa_info *isa,
                     uint32_t dw3, uint32_t dw2, uint32_t dw1, uint32_t dw0)
 {
    brw_inst inst;
    inst.data[0] = (((uint64_t) dw1) << 32) | ((uint64_t) dw0);
    inst.data[1] = (((uint64_t) dw3) << 32) | ((uint64_t) dw2);
-   return brw_disassemble_inst(stderr, devinfo, &inst, false, 0, NULL);
+   return brw_disassemble_inst(stderr, isa, &inst, false, 0, NULL);
 }
 #endif
 
@@ -1840,18 +2009,18 @@ lsc_disassemble_ex_desc(const struct intel_device_info *devinfo,
    const unsigned addr_type = lsc_msg_desc_addr_type(devinfo, imm_desc);
    switch (addr_type) {
    case LSC_ADDR_SURFTYPE_FLAT:
-      format(file, "base_offset %u ",
+      format(file, " base_offset %u ",
              lsc_flat_ex_desc_base_offset(devinfo, imm_ex_desc));
       break;
    case LSC_ADDR_SURFTYPE_BSS:
    case LSC_ADDR_SURFTYPE_SS:
-      format(file, "surface_state_index %u ",
+      format(file, " surface_state_index %u ",
              lsc_bss_ex_desc_index(devinfo, imm_ex_desc));
       break;
    case LSC_ADDR_SURFTYPE_BTI:
-      format(file, "BTI %u ",
+      format(file, " BTI %u ",
              lsc_bti_ex_desc_index(devinfo, imm_ex_desc));
-      format(file, "base_offset %u ",
+      format(file, " base_offset %u ",
              lsc_bti_ex_desc_base_offset(devinfo, imm_ex_desc));
       break;
    default:
@@ -1876,15 +2045,17 @@ brw_sfid_is_lsc(unsigned sfid)
 }
 
 int
-brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
+brw_disassemble_inst(FILE *file, const struct brw_isa_info *isa,
                      const brw_inst *inst, bool is_compacted,
                      int offset, const struct brw_label *root_label)
 {
+   const struct intel_device_info *devinfo = isa->devinfo;
+
    int err = 0;
    int space = 0;
 
-   const enum opcode opcode = brw_inst_opcode(devinfo, inst);
-   const struct opcode_desc *desc = brw_opcode_desc(devinfo, opcode);
+   const enum opcode opcode = brw_inst_opcode(isa, inst);
+   const struct opcode_desc *desc = brw_opcode_desc(isa, opcode);
 
    if (brw_inst_pred_control(devinfo, inst)) {
       string(file, "(");
@@ -1903,7 +2074,7 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
       string(file, ") ");
    }
 
-   err |= print_opcode(file, devinfo, opcode);
+   err |= print_opcode(file, isa, opcode);
 
    if (!is_send(opcode))
       err |= control(file, "saturate", saturate, brw_inst_saturate(devinfo, inst),
@@ -1922,7 +2093,19 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
       err |= control(file, "function", sync_function,
                      brw_inst_cond_modifier(devinfo, inst), NULL);
 
-   } else if (!is_send(opcode)) {
+   } else if (opcode == BRW_OPCODE_DPAS) {
+      string(file, ".");
+
+      err |= control(file, "systolic depth", dpas_systolic_depth,
+                     brw_inst_dpas_3src_sdepth(devinfo, inst), NULL);
+
+      const unsigned rcount = brw_inst_dpas_3src_rcount(devinfo, inst) + 1;
+
+      format(file, "x%d", rcount);
+   } else if (!is_send(opcode) &&
+              (devinfo->ver < 12 ||
+               brw_inst_src0_reg_file(devinfo, inst) != BRW_IMMEDIATE_VALUE ||
+               type_sz(brw_inst_src0_type(devinfo, inst)) < 8)) {
       err |= control(file, "conditional modifier", conditional_modifier,
                      brw_inst_cond_modifier(devinfo, inst), NULL);
 
@@ -1989,7 +2172,20 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
       format(file, "Pop: %"PRIu64, brw_inst_gfx4_pop_count(devinfo, inst));
    } else if (opcode == BRW_OPCODE_JMPI) {
       pad(file, 16);
-      err |= src1(file, devinfo, inst);
+      err |= src1(file, isa, inst);
+   } else if (opcode == BRW_OPCODE_DPAS) {
+      pad(file, 16);
+      err |= dest_dpas_3src(file, devinfo, inst);
+
+      pad(file, 32);
+      err |= src0_dpas_3src(file, devinfo, inst);
+
+      pad(file, 48);
+      err |= src1_dpas_3src(file, devinfo, inst);
+
+      pad(file, 64);
+      err |= src2_dpas_3src(file, devinfo, inst);
+
    } else if (desc && desc->nsrc == 3) {
       pad(file, 16);
       err |= dest_3src(file, devinfo, inst);
@@ -2005,17 +2201,17 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
    } else if (desc) {
       if (desc->ndst > 0) {
          pad(file, 16);
-         err |= dest(file, devinfo, inst);
+         err |= dest(file, isa, inst);
       }
 
       if (desc->nsrc > 0) {
          pad(file, 32);
-         err |= src0(file, devinfo, inst);
+         err |= src0(file, isa, inst);
       }
 
       if (desc->nsrc > 1) {
          pad(file, 48);
-         err |= src1(file, devinfo, inst);
+         err |= src1(file, isa, inst);
       }
    }
 
@@ -2049,7 +2245,7 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
          if (brw_inst_src1_reg_file(devinfo, inst) != BRW_IMMEDIATE_VALUE) {
             /* show the indirect descriptor source */
             pad(file, 48);
-            err |= src1(file, devinfo, inst);
+            err |= src1(file, isa, inst);
             pad(file, 64);
          } else {
             has_imm_desc = true;
@@ -2177,41 +2373,95 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
          }
 
          case BRW_SFID_URB: {
-            unsigned opcode = brw_inst_urb_opcode(devinfo, inst);
-
-            format(file, " offset %"PRIu64, brw_inst_urb_global_offset(devinfo, inst));
-
-            space = 1;
-
-            err |= control(file, "urb opcode",
-                           devinfo->ver >= 7 ? gfx7_urb_opcode
-                                             : gfx5_urb_opcode,
-                           opcode, &space);
-
-            if (devinfo->ver >= 7 &&
-                brw_inst_urb_per_slot_offset(devinfo, inst)) {
-               string(file, " per-slot");
-            }
-
-            if (opcode == GFX8_URB_OPCODE_SIMD8_WRITE ||
-                opcode == GFX8_URB_OPCODE_SIMD8_READ) {
-               if (brw_inst_urb_channel_mask_present(devinfo, inst))
-                  string(file, " masked");
-            } else if (opcode != GFX125_URB_OPCODE_FENCE) {
-               err |= control(file, "urb swizzle", urb_swizzle,
-                              brw_inst_urb_swizzle_control(devinfo, inst),
+            if (devinfo->ver >= 20) {
+               format(file, " (");
+               const enum lsc_opcode op = lsc_msg_desc_opcode(devinfo, imm_desc);
+               err |= control(file, "operation", lsc_operation,
+                              op, &space);
+               format(file, ",");
+               err |= control(file, "addr_size", lsc_addr_size,
+                              lsc_msg_desc_addr_size(devinfo, imm_desc),
                               &space);
-            }
 
-            if (devinfo->ver < 7) {
-               err |= control(file, "urb allocate", urb_allocate,
-                              brw_inst_urb_allocate(devinfo, inst), &space);
-               err |= control(file, "urb used", urb_used,
-                              brw_inst_urb_used(devinfo, inst), &space);
-            }
-            if (devinfo->ver < 8) {
-               err |= control(file, "urb complete", urb_complete,
-                              brw_inst_urb_complete(devinfo, inst), &space);
+               format(file, ",");
+               err |= control(file, "data_size", lsc_data_size,
+                              lsc_msg_desc_data_size(devinfo, imm_desc),
+                              &space);
+               format(file, ",");
+               if (lsc_opcode_has_cmask(op)) {
+                  err |= control(file, "component_mask",
+                                 lsc_cmask_str,
+                                 lsc_msg_desc_cmask(devinfo, imm_desc),
+                                 &space);
+               } else {
+                  err |= control(file, "vector_size",
+                                 lsc_vect_size_str,
+                                 lsc_msg_desc_vect_size(devinfo, imm_desc),
+                                 &space);
+                  if (lsc_msg_desc_transpose(devinfo, imm_desc))
+                     format(file, ", transpose");
+               }
+               switch(op) {
+               case LSC_OP_LOAD_CMASK:
+               case LSC_OP_LOAD:
+                  format(file, ",");
+                  err |= control(file, "cache_load",
+                                 lsc_cache_load,
+                                 lsc_msg_desc_cache_ctrl(devinfo, imm_desc),
+                                 &space);
+                  break;
+               default:
+                  format(file, ",");
+                  err |= control(file, "cache_store",
+                                 lsc_cache_store,
+                                 lsc_msg_desc_cache_ctrl(devinfo, imm_desc),
+                                 &space);
+                  break;
+               }
+
+               format(file, " dst_len = %u,", lsc_msg_desc_dest_len(devinfo, imm_desc));
+               format(file, " src0_len = %u,", lsc_msg_desc_src0_len(devinfo, imm_desc));
+               format(file, " src1_len = %d", brw_message_ex_desc_ex_mlen(devinfo, imm_ex_desc));
+               err |= control(file, "address_type", lsc_addr_surface_type,
+                              lsc_msg_desc_addr_type(devinfo, imm_desc), &space);
+               format(file, " )");
+            } else {
+               unsigned urb_opcode = brw_inst_urb_opcode(devinfo, inst);
+
+               format(file, " offset %"PRIu64, brw_inst_urb_global_offset(devinfo, inst));
+
+               space = 1;
+
+               err |= control(file, "urb opcode",
+                              devinfo->ver >= 7 ? gfx7_urb_opcode
+                              : gfx5_urb_opcode,
+                              urb_opcode, &space);
+
+               if (devinfo->ver >= 7 &&
+                   brw_inst_urb_per_slot_offset(devinfo, inst)) {
+                  string(file, " per-slot");
+               }
+
+               if (urb_opcode == GFX8_URB_OPCODE_SIMD8_WRITE ||
+                   urb_opcode == GFX8_URB_OPCODE_SIMD8_READ) {
+                  if (brw_inst_urb_channel_mask_present(devinfo, inst))
+                     string(file, " masked");
+               } else if (urb_opcode != GFX125_URB_OPCODE_FENCE) {
+                  err |= control(file, "urb swizzle", urb_swizzle,
+                                 brw_inst_urb_swizzle_control(devinfo, inst),
+                                 &space);
+               }
+
+               if (devinfo->ver < 7) {
+                  err |= control(file, "urb allocate", urb_allocate,
+                                 brw_inst_urb_allocate(devinfo, inst), &space);
+                  err |= control(file, "urb used", urb_used,
+                                 brw_inst_urb_used(devinfo, inst), &space);
+               }
+               if (devinfo->ver < 8) {
+                  err |= control(file, "urb complete", urb_complete,
+                                 brw_inst_urb_complete(devinfo, inst), &space);
+               }
             }
             break;
          }
@@ -2274,6 +2524,8 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
                case LSC_OP_LOAD:
                   format(file, ",");
                   err |= control(file, "cache_load",
+                                 devinfo->ver >= 20 ?
+                                 xe2_lsc_cache_load :
                                  lsc_cache_load,
                                  lsc_msg_desc_cache_ctrl(devinfo, imm_desc),
                                  &space);
@@ -2281,6 +2533,8 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
                default:
                   format(file, ",");
                   err |= control(file, "cache_store",
+                                 devinfo->ver >= 20 ?
+                                 xe2_lsc_cache_store :
                                  lsc_cache_store,
                                  lsc_msg_desc_cache_ctrl(devinfo, imm_desc),
                                  &space);
@@ -2289,7 +2543,11 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
             }
             format(file, " dst_len = %u,", lsc_msg_desc_dest_len(devinfo, imm_desc));
             format(file, " src0_len = %u,", lsc_msg_desc_src0_len(devinfo, imm_desc));
-            format(file, " src1_len = %d", brw_message_ex_desc_ex_mlen(devinfo, imm_ex_desc));
+
+            if (!brw_inst_send_sel_reg32_ex_desc(devinfo, inst))
+               format(file, " src1_len = %d",
+                      brw_message_ex_desc_ex_mlen(devinfo, imm_ex_desc));
+
             err |= control(file, "address_type", lsc_addr_surface_type,
                            lsc_msg_desc_addr_type(devinfo, imm_desc), &space);
             format(file, " )");
@@ -2424,11 +2682,19 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
          if (space)
             string(file, " ");
       }
+      if (devinfo->verx10 >= 125 &&
+          brw_inst_send_sel_reg32_ex_desc(devinfo, inst) &&
+          brw_inst_send_ex_bso(devinfo, inst)) {
+         format(file, " src1_len = %u",
+                (unsigned) brw_inst_send_src1_len(devinfo, inst));
+
+         format(file, " ex_bso");
+      }
       if (brw_sfid_is_lsc(sfid)) {
             lsc_disassemble_ex_desc(devinfo, imm_desc, imm_ex_desc, file);
       } else {
          if (has_imm_desc)
-            format(file, "mlen %u", brw_message_desc_mlen(devinfo, imm_desc));
+            format(file, " mlen %u", brw_message_desc_mlen(devinfo, imm_desc));
          if (has_imm_ex_desc) {
             format(file, " ex_mlen %u",
                    brw_message_ex_desc_ex_mlen(devinfo, imm_ex_desc));
@@ -2472,7 +2738,7 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
       }
 
       if (devinfo->ver >= 12)
-         err |= swsb(file, devinfo, inst);
+         err |= swsb(file, isa, inst);
 
       err |= control(file, "compaction", cmpt_ctrl, is_compacted, &space);
       err |= control(file, "thread control", thread_ctrl,
@@ -2482,7 +2748,7 @@ brw_disassemble_inst(FILE *file, const struct intel_device_info *devinfo,
       if (has_branch_ctrl(devinfo, opcode)) {
          err |= control(file, "branch ctrl", branch_ctrl,
                         brw_inst_branch_control(devinfo, inst), &space);
-      } else if (devinfo->ver >= 6) {
+      } else if (devinfo->ver >= 6 && devinfo->ver < 20) {
          err |= control(file, "acc write control", accwr,
                         brw_inst_acc_wr_control(devinfo, inst), &space);
       }
