@@ -33,7 +33,7 @@
 #include "main/glthread.h"
 #include "main/context.h"
 #include "main/macros.h"
-#include "marshal_generated.h"
+#include "main/matrix.h"
 
 struct marshal_cmd_base
 {
@@ -48,8 +48,28 @@ struct marshal_cmd_base
    uint16_t cmd_size;
 };
 
-typedef uint32_t (*_mesa_unmarshal_func)(struct gl_context *ctx, const void *cmd, const uint64_t *last);
+/* This must be included after "struct marshal_cmd_base" because it uses it. */
+#include "marshal_generated.h"
+
+typedef uint32_t (*_mesa_unmarshal_func)(struct gl_context *ctx,
+                                         const void *restrict cmd);
 extern const _mesa_unmarshal_func _mesa_unmarshal_dispatch[NUM_DISPATCH_CMD];
+extern const char *_mesa_unmarshal_func_name[NUM_DISPATCH_CMD];
+
+struct marshal_cmd_DrawElementsUserBuf
+{
+   struct marshal_cmd_base cmd_base;
+   GLenum16 mode;
+   GLenum16 type;
+   GLsizei count;
+   GLsizei instance_count;
+   GLint basevertex;
+   GLuint baseinstance;
+   GLuint drawid;
+   GLuint user_buffer_mask;
+   const GLvoid *indices;
+   struct gl_buffer_object *index_buffer;
+};
 
 static inline void *
 _mesa_glthread_allocate_command(struct gl_context *ctx,
@@ -73,6 +93,28 @@ _mesa_glthread_allocate_command(struct gl_context *ctx,
    return cmd_base;
 }
 
+static inline struct marshal_cmd_base *
+_mesa_glthread_get_cmd(uint64_t *opaque_cmd)
+{
+   return (struct marshal_cmd_base*)opaque_cmd;
+}
+
+static inline uint64_t *
+_mesa_glthread_next_cmd(uint64_t *opaque_cmd, unsigned cmd_size)
+{
+   assert(_mesa_glthread_get_cmd(opaque_cmd)->cmd_size == cmd_size);
+   return opaque_cmd + cmd_size;
+}
+
+static inline bool
+_mesa_glthread_call_is_last(struct glthread_state *glthread,
+                            struct marshal_cmd_base *last)
+{
+   return last &&
+          (uint64_t*)last + last->cmd_size ==
+          &glthread->next_batch->buffer[glthread->used];
+}
+
 static inline bool
 _mesa_glthread_has_no_pack_buffer(const struct gl_context *ctx)
 {
@@ -85,58 +127,6 @@ _mesa_glthread_has_no_unpack_buffer(const struct gl_context *ctx)
    return ctx->GLThread.CurrentPixelUnpackBufferName == 0;
 }
 
-/**
- * Instead of conditionally handling marshaling immediate index data in draw
- * calls (deprecated and removed in GL core), we just disable threading.
- */
-static inline bool
-_mesa_glthread_has_non_vbo_vertices_or_indices(const struct gl_context *ctx)
-{
-   const struct glthread_state *glthread = &ctx->GLThread;
-   struct glthread_vao *vao = glthread->CurrentVAO;
-
-   return ctx->API != API_OPENGL_CORE &&
-          (vao->CurrentElementBufferName == 0 ||
-           (vao->UserPointerMask & vao->BufferEnabled));
-}
-
-static inline bool
-_mesa_glthread_has_non_vbo_vertices(const struct gl_context *ctx)
-{
-   const struct glthread_state *glthread = &ctx->GLThread;
-   const struct glthread_vao *vao = glthread->CurrentVAO;
-
-   return ctx->API != API_OPENGL_CORE &&
-          (vao->UserPointerMask & vao->BufferEnabled);
-}
-
-static inline bool
-_mesa_glthread_has_non_vbo_vertices_or_indirect(const struct gl_context *ctx)
-{
-   const struct glthread_state *glthread = &ctx->GLThread;
-   const struct glthread_vao *vao = glthread->CurrentVAO;
-
-   return ctx->API != API_OPENGL_CORE &&
-          (glthread->CurrentDrawIndirectBufferName == 0 ||
-           (vao->UserPointerMask & vao->BufferEnabled));
-}
-
-static inline bool
-_mesa_glthread_has_non_vbo_vertices_or_indices_or_indirect(const struct gl_context *ctx)
-{
-   const struct glthread_state *glthread = &ctx->GLThread;
-   struct glthread_vao *vao = glthread->CurrentVAO;
-
-   return ctx->API != API_OPENGL_CORE &&
-          (glthread->CurrentDrawIndirectBufferName == 0 ||
-           vao->CurrentElementBufferName == 0 ||
-           (vao->UserPointerMask & vao->BufferEnabled));
-}
-
-
-bool
-_mesa_create_marshal_tables(struct gl_context *ctx);
-
 static inline unsigned
 _mesa_buffer_enum_to_count(GLenum buffer)
 {
@@ -182,6 +172,9 @@ _mesa_tex_param_enum_to_count(GLenum pname)
    case GL_TEXTURE_MAX_ANISOTROPY_EXT:
    case GL_TEXTURE_LOD_BIAS:
    case GL_TEXTURE_TILING_EXT:
+   case GL_TEXTURE_SPARSE_ARB:
+   case GL_VIRTUAL_PAGE_SIZE_INDEX_ARB:
+   case GL_NUM_SPARSE_LEVELS_ARB:
       return 1;
    case GL_TEXTURE_CROP_RECT_OES:
    case GL_TEXTURE_SWIZZLE_RGBA:
@@ -449,8 +442,12 @@ _mesa_glthread_Enable(struct gl_context *ctx, GLenum cap)
    case GL_PRIMITIVE_RESTART_FIXED_INDEX:
       _mesa_glthread_set_prim_restart(ctx, cap, true);
       break;
-   case GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB:
-      _mesa_glthread_destroy(ctx, "Enable(DEBUG_OUTPUT_SYNCHRONOUS)");
+   case GL_BLEND:
+      ctx->GLThread.Blend = true;
+      break;
+   case GL_DEBUG_OUTPUT_SYNCHRONOUS:
+      _mesa_glthread_disable(ctx);
+      ctx->GLThread.DebugOutputSynchronous = true;
       break;
    case GL_DEPTH_TEST:
       ctx->GLThread.DepthTest = true;
@@ -458,6 +455,24 @@ _mesa_glthread_Enable(struct gl_context *ctx, GLenum cap)
    case GL_CULL_FACE:
       ctx->GLThread.CullFace = true;
       break;
+   case GL_LIGHTING:
+      ctx->GLThread.Lighting = true;
+      break;
+   case GL_POLYGON_STIPPLE:
+      ctx->GLThread.PolygonStipple = true;
+      break;
+   case GL_VERTEX_ARRAY:
+   case GL_NORMAL_ARRAY:
+   case GL_COLOR_ARRAY:
+   case GL_TEXTURE_COORD_ARRAY:
+   case GL_INDEX_ARRAY:
+   case GL_EDGE_FLAG_ARRAY:
+   case GL_FOG_COORDINATE_ARRAY:
+   case GL_SECONDARY_COLOR_ARRAY:
+   case GL_POINT_SIZE_ARRAY_OES:
+      _mesa_glthread_ClientState(ctx, NULL, _mesa_array_to_attrib(ctx, cap),
+                                 true);
+      break;
    }
 }
 
@@ -472,23 +487,60 @@ _mesa_glthread_Disable(struct gl_context *ctx, GLenum cap)
    case GL_PRIMITIVE_RESTART_FIXED_INDEX:
       _mesa_glthread_set_prim_restart(ctx, cap, false);
       break;
+   case GL_BLEND:
+      ctx->GLThread.Blend = false;
+      break;
    case GL_CULL_FACE:
       ctx->GLThread.CullFace = false;
       break;
+   case GL_DEBUG_OUTPUT_SYNCHRONOUS:
+      ctx->GLThread.DebugOutputSynchronous = false;
+      _mesa_glthread_enable(ctx);
+      break;
    case GL_DEPTH_TEST:
       ctx->GLThread.DepthTest = false;
       break;
+   case GL_LIGHTING:
+      ctx->GLThread.Lighting = false;
+      break;
+   case GL_POLYGON_STIPPLE:
+      ctx->GLThread.PolygonStipple = false;
+      break;
+   case GL_VERTEX_ARRAY:
+   case GL_NORMAL_ARRAY:
+   case GL_COLOR_ARRAY:
+   case GL_TEXTURE_COORD_ARRAY:
+   case GL_INDEX_ARRAY:
+   case GL_EDGE_FLAG_ARRAY:
+   case GL_FOG_COORDINATE_ARRAY:
+   case GL_SECONDARY_COLOR_ARRAY:
+   case GL_POINT_SIZE_ARRAY_OES:
+      _mesa_glthread_ClientState(ctx, NULL, _mesa_array_to_attrib(ctx, cap),
+                                 false);
+      break;
    }
 }
 
 static inline int
 _mesa_glthread_IsEnabled(struct gl_context *ctx, GLenum cap)
 {
+   /* This will generate GL_INVALID_OPERATION, as it should. */
+   if (ctx->GLThread.inside_begin_end)
+      return -1;
+
    switch (cap) {
+   case GL_BLEND:
+      return ctx->GLThread.Blend;
    case GL_CULL_FACE:
       return ctx->GLThread.CullFace;
+   case GL_DEBUG_OUTPUT_SYNCHRONOUS:
+      return ctx->GLThread.DebugOutputSynchronous;
    case GL_DEPTH_TEST:
       return ctx->GLThread.DepthTest;
+   case GL_LIGHTING:
+      return ctx->GLThread.Lighting;
+   case GL_POLYGON_STIPPLE:
+      return ctx->GLThread.PolygonStipple;
    case GL_VERTEX_ARRAY:
       return !!(ctx->GLThread.CurrentVAO->UserEnabled & VERT_BIT_POS);
    case GL_NORMAL_ARRAY:
@@ -517,12 +569,20 @@ _mesa_glthread_PushAttrib(struct gl_context *ctx, GLbitfield mask)
 
    attr->Mask = mask;
 
-   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT))
+   if (mask & GL_ENABLE_BIT)
+      attr->Blend = ctx->GLThread.Blend;
+
+   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT)) {
       attr->CullFace = ctx->GLThread.CullFace;
+      attr->PolygonStipple = ctx->GLThread.PolygonStipple;
+   }
 
    if (mask & (GL_DEPTH_BUFFER_BIT | GL_ENABLE_BIT))
       attr->DepthTest = ctx->GLThread.DepthTest;
 
+   if (mask & (GL_LIGHTING_BIT | GL_ENABLE_BIT))
+      attr->Lighting = ctx->GLThread.Lighting;
+
    if (mask & GL_TEXTURE_BIT)
       attr->ActiveTexture = ctx->GLThread.ActiveTexture;
 
@@ -543,12 +603,20 @@ _mesa_glthread_PopAttrib(struct gl_context *ctx)
       &ctx->GLThread.AttribStack[--ctx->GLThread.AttribStackDepth];
    unsigned mask = attr->Mask;
 
-   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT))
+   if (mask & GL_ENABLE_BIT)
+      ctx->GLThread.Blend = attr->Blend;
+
+   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT)) {
       ctx->GLThread.CullFace = attr->CullFace;
+      ctx->GLThread.PolygonStipple = attr->PolygonStipple;
+   }
 
    if (mask & (GL_DEPTH_BUFFER_BIT | GL_ENABLE_BIT))
       ctx->GLThread.DepthTest = attr->DepthTest;
 
+   if (mask & (GL_LIGHTING_BIT | GL_ENABLE_BIT))
+      ctx->GLThread.Lighting = attr->Lighting;
+
    if (mask & GL_TEXTURE_BIT)
       ctx->GLThread.ActiveTexture = attr->ActiveTexture;
 
@@ -645,7 +713,7 @@ _mesa_glthread_MatrixMode(struct gl_context *ctx, GLenum mode)
       return;
 
    ctx->GLThread.MatrixIndex = _mesa_get_matrix_index(ctx, mode);
-   ctx->GLThread.MatrixMode = mode;
+   ctx->GLThread.MatrixMode = MIN2(mode, 0xffff);
 }
 
 static inline void
@@ -657,6 +725,39 @@ _mesa_glthread_ListBase(struct gl_context *ctx, GLuint base)
    ctx->GLThread.ListBase = base;
 }
 
+static inline void
+_mesa_glthread_init_call_fence(int *last_batch_index_where_called)
+{
+   *last_batch_index_where_called = -1;
+}
+
+static inline void
+_mesa_glthread_fence_call(struct gl_context *ctx,
+                          int *last_batch_index_where_called)
+{
+   p_atomic_set(last_batch_index_where_called, ctx->GLThread.next);
+   /* Flush, so that the fenced call is last in the batch. */
+   _mesa_glthread_flush_batch(ctx);
+}
+
+static inline void
+_mesa_glthread_signal_call(int *last_batch_index_where_called, int batch_index)
+{
+   /* Atomically set this to -1 if it's equal to batch_index. */
+   p_atomic_cmpxchg(last_batch_index_where_called, batch_index, -1);
+}
+
+static inline void
+_mesa_glthread_wait_for_call(struct gl_context *ctx,
+                             int *last_batch_index_where_called)
+{
+   int batch = p_atomic_read(last_batch_index_where_called);
+   if (batch != -1) {
+      util_queue_fence_wait(&ctx->GLThread.batches[batch].fence);
+      assert(p_atomic_read(last_batch_index_where_called) == -1);
+   }
+}
+
 static inline void
 _mesa_glthread_CallList(struct gl_context *ctx, GLuint list)
 {
@@ -667,11 +768,7 @@ _mesa_glthread_CallList(struct gl_context *ctx, GLuint list)
     * all display lists are up to date and the driver thread is not
     * modifiying them. We will be executing them in the application thread.
     */
-   int batch = p_atomic_read(&ctx->GLThread.LastDListChangeBatchIndex);
-   if (batch != -1) {
-      util_queue_fence_wait(&ctx->GLThread.batches[batch].fence);
-      p_atomic_set(&ctx->GLThread.LastDListChangeBatchIndex, -1);
-   }
+   _mesa_glthread_wait_for_call(ctx, &ctx->GLThread.LastDListChangeBatchIndex);
 
    if (!ctx->Shared->DisplayListsAffectGLThread)
       return;
@@ -699,11 +796,7 @@ _mesa_glthread_CallLists(struct gl_context *ctx, GLsizei n, GLenum type,
     * all display lists are up to date and the driver thread is not
     * modifiying them. We will be executing them in the application thread.
     */
-   int batch = p_atomic_read(&ctx->GLThread.LastDListChangeBatchIndex);
-   if (batch != -1) {
-      util_queue_fence_wait(&ctx->GLThread.batches[batch].fence);
-      p_atomic_set(&ctx->GLThread.LastDListChangeBatchIndex, -1);
-   }
+   _mesa_glthread_wait_for_call(ctx, &ctx->GLThread.LastDListChangeBatchIndex);
 
    /* Clear GL_COMPILE_AND_EXECUTE if needed. We only execute here. */
    unsigned saved_mode = ctx->GLThread.ListMode;
@@ -788,10 +881,10 @@ _mesa_glthread_CallLists(struct gl_context *ctx, GLsizei n, GLenum type,
 }
 
 static inline void
-_mesa_glthread_NewList(struct gl_context *ctx, GLuint list, GLuint mode)
+_mesa_glthread_NewList(struct gl_context *ctx, GLuint list, GLenum mode)
 {
    if (!ctx->GLThread.ListMode)
-      ctx->GLThread.ListMode = mode;
+      ctx->GLThread.ListMode = MIN2(mode, 0xffff);
 }
 
 static inline void
@@ -803,8 +896,7 @@ _mesa_glthread_EndList(struct gl_context *ctx)
    ctx->GLThread.ListMode = 0;
 
    /* Track the last display list change. */
-   p_atomic_set(&ctx->GLThread.LastDListChangeBatchIndex, ctx->GLThread.next);
-   _mesa_glthread_flush_batch(ctx);
+   _mesa_glthread_fence_call(ctx, &ctx->GLThread.LastDListChangeBatchIndex);
 }
 
 static inline void
@@ -814,8 +906,24 @@ _mesa_glthread_DeleteLists(struct gl_context *ctx, GLsizei range)
       return;
 
    /* Track the last display list change. */
-   p_atomic_set(&ctx->GLThread.LastDListChangeBatchIndex, ctx->GLThread.next);
-   _mesa_glthread_flush_batch(ctx);
+   _mesa_glthread_fence_call(ctx, &ctx->GLThread.LastDListChangeBatchIndex);
+}
+
+static inline void
+_mesa_glthread_BindFramebuffer(struct gl_context *ctx, GLenum target, GLuint id)
+{
+   switch (target) {
+   case GL_FRAMEBUFFER:
+      ctx->GLThread.CurrentDrawFramebuffer = id;
+      ctx->GLThread.CurrentReadFramebuffer = id;
+      break;
+   case GL_DRAW_FRAMEBUFFER:
+      ctx->GLThread.CurrentDrawFramebuffer = id;
+      break;
+   case GL_READ_FRAMEBUFFER:
+      ctx->GLThread.CurrentReadFramebuffer = id;
+      break;
+   }
 }
 
 static inline void
@@ -824,18 +932,12 @@ _mesa_glthread_DeleteFramebuffers(struct gl_context *ctx, GLsizei n,
 {
    if (ctx->GLThread.CurrentDrawFramebuffer) {
       for (int i = 0; i < n; i++) {
-         if (ctx->GLThread.CurrentDrawFramebuffer == ids[i]) {
+         if (ctx->GLThread.CurrentDrawFramebuffer == ids[i])
             ctx->GLThread.CurrentDrawFramebuffer = 0;
-            break;
-         }
+         if (ctx->GLThread.CurrentReadFramebuffer == ids[i])
+            ctx->GLThread.CurrentReadFramebuffer = 0;
       }
    }
 }
 
-struct marshal_cmd_CallList
-{
-   struct marshal_cmd_base cmd_base;
-   GLuint list;
-};
-
 #endif /* MARSHAL_H */
